###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['RetireWdg', 'InfoWdg']

from pyasm.common import Date, Container
from pyasm.web import *
from pyasm.search import Search
from pyasm.widget import FilterSelectWdg, SelectWdg, IconRefreshWdg, ThumbWdg, ReactivateLinkWdg
from pyasm.prod.web import DateFilterWdg


class RetireWdg(Widget):

    def init(my):

        WebContainer.register_cmd("pyasm.command.ReactivateCmd")

        # add a time filter
        div = DivWdg(css='filter_box')
        div.add_style("text-align: center")
        div.add_style("padding: 5px")

        date_filter = DateFilterWdg(name='time_filter')
        # this means last 2 days
        date_filter.set_option("default", "1 Day")
        div.add(date_filter)

        my.add(div)

        search = Search("sthpw/retire_log")
        date_filter.alter_search(search) 
        
        search.add_order_by("timestamp desc")
        user = WebEnvironment.get_user_name()
        search.add_filter("login", user)

        retire_logs = search.get_sobjects()

        widget = Widget()


        table = Table()
        table.set_class("table")
        table.set_max_width()

        table.add_row()
        table.add_header("Icon")
        table.add_header("Info")
        table.add_header("Date Retired")
        table.add_header("User")
        table.add_header("&nbsp;")


        for retire_log in retire_logs:

            search_type = retire_log.get_value("search_type")
            search_id = retire_log.get_value("search_id")
            timestamp = retire_log.get_value("timestamp")

            sobject = None
            try: 
                sobject = Search.get_by_id(search_type,search_id)
            except:
                table.add_row_cell("<h2>Error!!!: id: %s" % retire_log.get_id())
                continue

            if not sobject:
                table.add_row_cell("<h2>Error!!!: id: %s" % retire_log.get_id())
                continue


            tr = table.add_row()
            tr.set_id("table_row_%s" % search_id )

            # try to guess the icon attr
            thumb = ThumbWdg()
            #if sobject.has_value("images"):
            #    thumb.set_name("images")
            #    thumb.set_sobject(sobject)
            #elif sobject.has_value("files"):
            #    thumb.set_name("files")
            #    thumb.set_sobject(sobject)
            #else:
            #    thumb = "&nbsp;"
            thumb.set_sobject(sobject)
            table.add_cell( thumb )


            info_wdg = InfoWdg()
            info_wdg.set_sobject(sobject)
            table.add_cell(info_wdg)


            date = Date( db=timestamp ).get_display_time()
            table.add_cell( date )

            table.add_cell( retire_log.get_value("login") )
            reactivate = ReactivateLinkWdg(search_type,search_id, sobject.get_code())
            table.add_cell(reactivate)


        widget.add(table)



        my.add(widget)




class InfoWdg(Widget):

    def get_display(my):

        sobject = my.get_current_sobject()
        attr_names = sobject.get_attr_names()
        attr_table = Table()
        attr_table.set_class("embed")

        excluded_attr_names = ['id','s_status','files','images']

        for attr_name in attr_names:
            if attr_name in excluded_attr_names:
                continue
            attr_table.add_row()
            attr_table.add_header(attr_name)
            attr_table.add_cell(sobject.get_value(attr_name))

        my.add(attr_table)

        return super(InfoWdg,my).get_display()







