###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['TextInputWdg', 'LookAheadTextInputWdg']

from pyasm.common import Date, Common
from pyasm.web import Table, DivWdg, SpanWdg, WebContainer, Widget
from pyasm.biz import Project
from pyasm.search import Search
from pyasm.widget import IconWdg, TextWdg, BaseInputWdg
from tactic.ui.common import BaseRefreshWdg


import random


class TextInputWdg(BaseInputWdg):

    ARGS_KEYS = {
    'width': {
        'description': 'width of the text element',
        'type': 'TextWdg',
        'order': 1,
        'category': 'Options'
    },
    'hint_text': {
        'description': 'initial hint text',
        'type': 'TextWdg',
        'order': 2,
        'category': 'Options'
        }
    }
 




    def __init__(my, **kwargs):
        name = kwargs.get("name")
       

        my.text = TextWdg(name)
        my.text.add_class("spt_text_input")
        border_color = my.text.get_color("border")
        my.text.add_style("border: solid 1px %s" % border_color)
        my.text.add_style("padding: 4px")
        my.text.add_color("background", "background", -3)

       
        my.top = SpanWdg()

        my.kwargs = kwargs
        super(TextInputWdg, my).__init__()


        my.width = my.kwargs.get("width")
        if not my.width:
            my.width = 200
        else:
            my.width = my.width.replace("px", "")
            my.width = int(my.width)

        my.readonly = False


    def add_style(my, name, value=None):
        if not value:
            name, value = name.split(": ")

        if name == 'width':
            my.width = int(value.replace("px",""))
        elif name == 'float':
            my.top.add_style(name, value)
        else:
            my.text.add_style(name, value)


    def add_behavior(my, behavior):
        my.text.add_behavior(behavior)





    def set_readonly(my, flag=True):
        my.readonly = flag
        my.text.set_attr("readonly", "readonly")

    def add_class(my, class_name):
        my.text.add_class(class_name)

    def add_color(my, name, palette_key, modifier=0, default=None):
        my.text.add_color(name, palette_key, modifier, default)

    def set_attr(my, name, value):
        my.text.set_attr(name, value)

    def set_value(my, value):
        my.text.set_value(value)
        

    def get_display(my):

        top = my.top
        top.add_style("position: relative")
        top.add_class("spt_text_top")
        top.add_style("height: 20px")
        top.add_style("width: %spx" % my.width)
        top.add_class("spt_input_text_top")

        text_div = SpanWdg()
        top.add(text_div)
        text_div.add(my.text)
        
        if not my.text.value:
            hint_text = my.kwargs.get("hint_text")
            if hint_text:
                my.text.add_attr('title', hint_text)
                # this prevents using this value for search
                my.text.add_behavior({ 'type': 'load',
                    'cbjs_action': '''
                    var over = new OverText(bvr.src_el, {
                        positionOptions: {
                            offset: {x:5, y:4}}});
                    over.text.setStyle('color','#999');
                    over.text.setStyle('font-size','11px');
                    over.text.setStyle('font-family','Arial, Serif');
                    '''})

		

        #my.text.add_style("-moz-border-radius: 5px")
        my.text.set_round_corners()
        my.text.add_style("width: %spx" % my.width)
        text_div.add_style("width: %spx" % my.width)

        icon_wdg = DivWdg()
        text_div.add(icon_wdg)
        icon_wdg.add_style("position: absolute")
        
        if WebContainer.get_web().get_browser() == 'Webkit':
            top_offset = '2'
            right_offset = '6'
        else:
            top_offset = '1'
            right_offset = '10'

        icon_wdg.add_style("top: %spx" %top_offset)
        icon_wdg.add_style("right: %spx" % right_offset)


        if not my.readonly:
            icon = IconWdg("Clear", IconWdg.CLOSE_INACTIVE)
            icon.add_class("spt_icon_inactive")
            icon_wdg.add(icon)
            icon = IconWdg("Clear", IconWdg.CLOSE_ACTIVE)
            icon.add_class("spt_icon_active")
            icon.add_style("display: none")
            icon_wdg.add(icon)

            icon_wdg.add_behavior( {
            'type': 'hover',
            'cbjs_action_over': '''
            var inactive = bvr.src_el.getElement(".spt_icon_inactive");
            var active = bvr.src_el.getElement(".spt_icon_active");
            spt.show(active);
            spt.hide(inactive);
            ''',
            'cbjs_action_out': '''
            var inactive = bvr.src_el.getElement(".spt_icon_inactive");
            var active = bvr.src_el.getElement(".spt_icon_active");
            spt.show(inactive);
            spt.hide(active);
            '''
            })



            icon.add_behavior( {
                'type': 'click_up',
                'cbjs_action': '''
                var top = bvr.src_el.getParent(".spt_text_top");
                var text = top.getElement(".spt_input");
                text.value = '';
                '''
            } )

        top.add("&nbsp")
        return top





class LookAheadTextInputWdg(TextInputWdg):

    def init(my):
        my.add_behavior( {
            'type': 'load',
            'cbjs_action': '''
spt.text_input = {}

spt.text_input.is_on = false;
spt.text_input.index = -1;
spt.text_input.last_index = 0;
            '''
        } )
       
        my.text.add_behavior( {
            'type': 'blur',
            'cbjs_action': '''
            // put a delay in here so that a click in the results
            // has time to register
            setTimeout( function() {
                var top = bvr.src_el.getParent(".spt_input_text_top");
                var el = top.getElement(".spt_input_text_results");
                el.setStyle("display", "none");

                spt.text_input.last_index = 0;
                spt.text_input.index = -1;
            }, 250 )
            '''
        } )

        

        results_class_name = my.kwargs.get("results_class_name")
        if not results_class_name:
            results_class_name = 'tactic.ui.input.TextInputResultsWdg';


        custom_cbk = my.kwargs.get("custom_cbk")
        if not custom_cbk:
            custom_cbk = {}


        search_type = my.kwargs.get("search_type")
        filter_search_type = my.kwargs.get("filter_search_type")
        if not search_type:
            search_type = 'sthpw/sobject_list'
        column = my.kwargs.get("column")
        if not column:
            column = 'keywords'

        if not custom_cbk.get('enter'):
            # this is meant for the Project startup page
            custom_cbk['enter'] = '''
                var top = bvr.src_el.getParent(".spt_main_top");
                var search_el = top.getElement(".spt_main_search");
                var keywords = search_el.value;
               

                var class_name = 'tactic.ui.panel.ViewPanelWdg';
                var kwargs = {
                    'search_type': '%s',
                    'filter_search_type': '%s',
                    'view': 'table',
                    'keywords': keywords,
                    'simple_search_view': 'simple_search',
                    //'show_shelf': false,
                }
                spt.tab.set_main_body_tab();
                spt.tab.add_new("Search", "Search", class_name, kwargs);
            ''' % (search_type, filter_search_type)


        bgcolor = my.text.get_color("background3")
      
        my.text.add_behavior( {
            'type': 'keyup',
            'custom': custom_cbk,
            'search_type': search_type,
            'filter_search_type': filter_search_type,
            'column': column,
            'results_class_name': results_class_name,
            'bg_color': bgcolor,
            'cbjs_action': '''
            var key = evt.key;
            try {
                 
                if (key == 'down') {
                    var top = bvr.src_el.getParent(".spt_input_text_top");
                    var els = top.getElements(".spt_input_text_result");
                    spt.text_input.last_index = spt.text_input.index;
                    spt.text_input.index += 1;
                    // index is redundant, just to make variable shorter
                    var index = spt.text_input.index;
                    // rewind
                    if (index == els.length) {
                        spt.text_input.index = 0;
                        index = 0;
                    }
                    
                    if (spt.text_input.last_index >=0)
                        els[spt.text_input.last_index].setStyle('background','');
                    if (els && els.length > 0)
                        els[index].setStyle('background',bvr.bg_color);
                    return;
                }
                else if (key == 'up') {
                    var top = bvr.src_el.getParent(".spt_input_text_top");
                    var els = top.getElements(".spt_input_text_result");
                    spt.text_input.last_index = spt.text_input.index;
                    spt.text_input.index -= 1;
                    var index = spt.text_input.index;
                    if (index < 0) {
                        index = els.length - 1;
                        spt.text_input.index = index;
                    }
                   

                    if (0 <= spt.text_input.last_index <= els.length-1) {
                        els[spt.text_input.last_index].setStyle('background','');
                    }
                    if (els) 
                        els[index].setStyle('background',bvr.bg_color);
                    return;
                }
                else if (key == 'enter') {

                    var top = bvr.src_el.getParent(".spt_input_text_top");
                    var els = top.getElements(".spt_input_text_result");
                    if (els && spt.text_input.index > -1)
                        bvr.src_el.value = els[spt.text_input.index].getAttribute('spt_value');

                    var custom = bvr.custom.enter;
                    if (custom) {
                        eval(custom);
                    }
                    var res_top = top.getElement(".spt_input_text_results");
                    spt.hide(res_top);
                    
                    return;
                }
                else if (key == 'esc') {
                    var top = bvr.src_el.getParent(".spt_input_text_top");
                    var res_top = top.getElement(".spt_input_text_results");
                    spt.hide(res_top);
                    return;
                }
                else if (key == 'left'|| key == 'right') {
                   return;
                }
            }
            catch(e) {
                spt.text_input.last_index = -1;
                spt.text_input.index = 0;
            }

            var value = bvr.src_el.value;
            if (value == '') {
                return;
            }

            var class_name = bvr.results_class_name;

            var cbk = function(html) {
                var top = bvr.src_el.getParent(".spt_input_text_top");
                var el = top.getElement(".spt_input_text_results");
                el.setStyle("display", "");
                el.innerHTML = html;

                spt.text_input.is_on = false;
            }
            var kwargs = {
                args: {
                    search_type: bvr.search_type,
                    filter_search_type: bvr.filter_search_type,
                    column: bvr.column,
                    value: value
                },
                cbjs_action: cbk,
            }

            var server = TacticServerStub.get();
            if (spt.text_input.is_on == false) {
                spt.text_input.is_on = true;
                server.async_get_widget(class_name, kwargs);
            }
            '''
        } )



        results_div = DivWdg()
        my.top.add(results_div)
        results_div.add_style("display: none")
        results_div.add_style("position: absolute")
        results_div.add_style("top: 25px")
        results_div.add_style("left: 10px")
        results_div.add_color("background", "background")
        results_div.add_color("color", "color")
        results_div.add_style("padding: 5px 10px 10px 5px")
        results_div.add_style("min-width: 220px")
        results_div.add_style("z-index: 1000")
        results_div.add_style("font-size: 16px")
        results_div.add_style("font-weight: bold")
        results_div.add_border()
        results_div.set_box_shadow('1px 2px 2px 2px')
        results_div.add_class("spt_input_text_results")

        bgcolor = results_div.get_color("background3")
        results_div.add_relay_behavior( {
            'type': "mouseover",
            'bgcolor': bgcolor,
            'bvr_match_class': 'spt_input_text_result',
            'cbjs_action': '''
            bvr.src_el.setStyle("background", bvr.bgcolor);
            '''
        } )
        results_div.add_relay_behavior( {
            'type': "mouseout",
            'bvr_match_class': 'spt_input_text_result',
            'cbjs_action': '''
            bvr.src_el.setStyle("background", "");
            '''
        } )
        results_div.add_relay_behavior( {
            'type': "mouseup",
            'bvr_match_class': 'spt_input_text_result',
            'cbjs_action': '''
            var value = bvr.src_el.getAttribute("spt_value");

            var top = bvr.src_el.getParent(".spt_input_text_top");
            var el = top.getElement(".spt_input_text_results");
            el.setStyle("display", "none");

            var text_el = top.getElement(".spt_text_input");
            text_el.value = value;


            '''
        } )


__all__.append("TextInputResultsWdg")
class TextInputResultsWdg(BaseRefreshWdg):

    def get_display(my):
        top = my.top
        value = my.kwargs.get("value")
        if value.endswith(" "):
            last_word_complete = True
        else:
            last_word_complete = False


        search_type = my.kwargs.get("search_type")
        filter_search_type = my.kwargs.get("filter_search_type")
        
        column = my.kwargs.get("column")

        if not search_type:
            search_type = "sthpw/sobject_list"
        if not column:
            column = "keywords"


        if isinstance(column, basestring):
            columns = [column]
        else:
            columns = column


        value = value.strip()
        values = Common.extract_keywords(value)
        values.reverse()


        project_code = Project.get_project_code()

        search = Search(search_type)
        #search.add_text_search_filter(column, values)
        if search_type == 'sthpw/sobject_list':
            search.add_filter("project_code", project_code)

        if search_type == 'sthpw/sobject_list' and filter_search_type and filter_search_type != 'None':
            search.add_filter("search_type", filter_search_type)
        
        search.add_op("begin")

        for column in columns:
            search.add_startswith_keyword_filter(column, values)
        
        search.add_op("or")

        search.add_limit(10)
        #print search.get_statement()
        results = search.get_sobjects()

        max = 35

        # English: ???
        ignore = set()
        ignore.add("of")
        ignore.add("in")
        ignore.add("the")
        ignore.add("for")
        ignore.add("a")
        ignore.add("and")

        filtered = []
        first_filtered = []
        second_filtered = []
        for result in results:
            keywords = []
            for column in columns:
                keywords.append(result.get_value(column))
            keywords = " ".join(keywords)
            keywords = keywords.lower()
            


            # show the keyword that matched first
            keywords = keywords.split(" ")
            #keywords_set = set()
            #for keyword in keywords:
            #    keywords_set.add(keyword)
            keywords = filter(None, keywords)
           
            matches = []
            for i, value in enumerate(values):
                for keyword in keywords:
                    # only the last is compared with start
                    if i == 0 and not last_word_complete:
                        compare = keyword.startswith(value)
                    else:
                        compare = keyword == value

                    if compare:
                        matches.append(keyword)
                        break

            if len(matches) != len(values):
                continue

            for match in matches:
                keywords.remove(match)
                keywords.insert(0, match)

            # get the first match
            first = keywords[:len(matches)]
            first = filter(None, first)
            first = " ".join(first)
            if first not in first_filtered:
                first_filtered.append(first)

            # get all the other keywords
            for second in keywords[len(matches):]:
                if first == second:
                    continue
                key = "%s %s" % (first, second)
                if key not in second_filtered:
                    second_filtered.append(key)


        first_filtered.sort()
        filtered.extend(first_filtered)
        second_filtered.sort()
        filtered.extend(second_filtered)

        filtered = filtered[0:10]


        for keywords in filtered:
            div = DivWdg()
            div.add_style('font-size: 10px')
            top.add(div)

            if len(keywords) > max:
                display = "%s..." % keywords[:max-3]
            else:
                display = keywords


            div.add(display)
            div.add_class("spt_input_text_result")
            div.add_attr("spt_value", keywords)



        return top




